@@ -26,11 +26,6 @@
 #ifndef PSYCOPG_ASIS_H
 #define PSYCOPG_ASIS_H 1
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-
-#include "psycopg/config.h"
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -45,12 +40,6 @@ typedef struct {
 
 } asisObject;
 
-/* functions exported to psycopgmodule.c */
-
-HIDDEN PyObject *psyco_AsIs(PyObject *module, PyObject *args);
-#define psyco_AsIs_doc \
-    "AsIs(obj) -> new AsIs wrapper object"
-
 #ifdef __cplusplus
 }
 #endif
