@@ -23,25 +23,18 @@
  * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <stringobject.h>
-
-#include <libpq-fe.h>
-#include <string.h>
-
 #define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
 #include "psycopg/psycopg.h"
-#include "psycopg/connection.h"
+
 #include "psycopg/adapter_binary.h"
 #include "psycopg/microprotocols_proto.h"
+#include "psycopg/connection.h"
+
+#include <string.h>
+
 
 /** the quoting code */
 
-#ifndef PSYCOPG_OWN_QUOTING
 static unsigned char *
 binary_escape(unsigned char *from, size_t from_length,
                size_t *to_length, PGconn *conn)
@@ -53,161 +46,120 @@ binary_escape(unsigned char *from, size_t from_length,
 #endif
         return PQescapeBytea(from, from_length, to_length);
 }
-#else
-static unsigned char *
-binary_escape(unsigned char *from, size_t from_length,
-               size_t *to_length, PGconn *conn)
-{
-    unsigned char *quoted, *chptr, *newptr;
-    size_t i, space, new_space;
 
-    space = from_length + 2;
-
-    Py_BEGIN_ALLOW_THREADS;
-
-    quoted = (unsigned char*)calloc(space, sizeof(char));
-    if (quoted == NULL) return NULL;
-
-    chptr = quoted;
-
-    for (i = 0; i < from_length; i++) {
-        if (chptr - quoted > space - 6) {
-            new_space  =  space * ((space) / (i + 1)) + 2 + 6;
-            if (new_space - space < 1024) space += 1024;
-            else space = new_space;
-            newptr = (unsigned char *)realloc(quoted, space);
-            if (newptr == NULL) {
-                free(quoted);
-                return NULL;
-            }
-            /* chptr has to be moved to the new location*/
-            chptr = newptr + (chptr - quoted);
-            quoted = newptr;
-            Dprintf("binary_escape: reallocated %i bytes at %p", space,quoted);
-        }
-        if (from[i]) {
-            if (from[i] >= ' ' && from[i] <= '~') {
-                if (from[i] == '\'') {
-                    *chptr = '\'';
-                    chptr++;
-                    *chptr = '\'';
-                    chptr++;
-                }
-                else if (from[i] == '\\') {
-                    memcpy(chptr, "\\\\\\\\", 4);
-                    chptr += 4;
-                }
-                else {
-                    /* leave it as it is if ascii printable */
-                    *chptr = from[i];
-                    chptr++;
-                }
-            }
-            else {
-                unsigned char c;
-
-                /* escape to octal notation \nnn */
-                *chptr++ = '\\';
-                *chptr++ = '\\';
-                c = from[i];
-                *chptr = ((c >> 6) & 0x07) + 0x30; chptr++;
-                *chptr = ((c >> 3) & 0x07) + 0x30; chptr++;
-                *chptr = ( c       & 0x07) + 0x30; chptr++;
-            }
-        }
-        else {
-            /* escape null as \\000 */
-            memcpy(chptr, "\\\\000", 5);
-            chptr += 5;
-        }
-    }
-    *chptr = '\0';
-
-    Py_END_ALLOW_THREADS;
-
-    *to_length = chptr - quoted + 1;
-    return quoted;
-}
-#endif
+#define HAS_BUFFER (PY_MAJOR_VERSION < 3)
+#define HAS_MEMORYVIEW (PY_MAJOR_VERSION > 2 || PY_MINOR_VERSION >= 6)
 
 /* binary_quote - do the quote process on plain and unicode strings */
 
 static PyObject *
 binary_quote(binaryObject *self)
 {
-    char *to;
-    const char *buffer;
+    char *to = NULL;
+    const char *buffer = NULL;
     Py_ssize_t buffer_len;
     size_t len = 0;
+    PyObject *rv = NULL;
+#if HAS_MEMORYVIEW
+    Py_buffer view;
+    int got_view = 0;
+#endif
+
+    /* Allow Binary(None) to work */
+    if (self->wrapped == Py_None) {
+        Py_INCREF(psyco_null);
+        rv = psyco_null;
+        goto exit;
+    }
 
     /* if we got a plain string or a buffer we escape it and save the buffer */
-    if (PyString_Check(self->wrapped) || PyBuffer_Check(self->wrapped)) {
-        /* escape and build quoted buffer */
-        if (PyObject_AsReadBuffer(self->wrapped, (const void **)&buffer,
-                                  &buffer_len) < 0)
-            return NULL;
 
-        to = (char *)binary_escape((unsigned char*)buffer, (size_t) buffer_len,
-            &len, self->conn ? ((connectionObject*)self->conn)->pgconn : NULL);
-        if (to == NULL) {
-            PyErr_NoMemory();
-            return NULL;
+#if HAS_MEMORYVIEW
+    if (PyObject_CheckBuffer(self->wrapped)) {
+        if (0 > PyObject_GetBuffer(self->wrapped, &view, PyBUF_CONTIG_RO)) {
+            goto exit;
         }
+        got_view = 1;
+        buffer = (const char *)(view.buf);
+        buffer_len = view.len;
+    }
+#endif
 
-        if (len > 0)
-            self->buffer = PyString_FromFormat(
-                (self->conn && ((connectionObject*)self->conn)->equote)
-                    ? "E'%s'" : "'%s'" , to);
-        else
-            self->buffer = PyString_FromString("''");
+#if HAS_BUFFER
+    if (!buffer && (Bytes_Check(self->wrapped) || PyBuffer_Check(self->wrapped))) {
+        if (PyObject_AsReadBuffer(self->wrapped, (const void **)&buffer,
+                                  &buffer_len) < 0) {
+            goto exit;
+        }
+    }
+#endif
 
-        PQfreemem(to);
+    if (!buffer) {
+        goto exit;
     }
 
-    /* if the wrapped object is not a string or a buffer, this is an error */
-    else {
-        PyErr_SetString(PyExc_TypeError, "can't escape non-string object");
-        return NULL;
+    /* escape and build quoted buffer */
+
+    to = (char *)binary_escape((unsigned char*)buffer, (size_t)buffer_len,
+        &len, self->conn ? ((connectionObject*)self->conn)->pgconn : NULL);
+    if (to == NULL) {
+        PyErr_NoMemory();
+        goto exit;
     }
 
-    return self->buffer;
+    if (len > 0)
+        rv = Bytes_FromFormat(
+            (self->conn && ((connectionObject*)self->conn)->equote)
+                ? "E'%s'::bytea" : "'%s'::bytea" , to);
+    else
+        rv = Bytes_FromString("''::bytea");
+
+exit:
+    if (to) { PQfreemem(to); }
+#if HAS_MEMORYVIEW
+    if (got_view) { PyBuffer_Release(&view); }
+#endif
+
+    /* if the wrapped object is not bytes or a buffer, this is an error */
+    if (!rv && !PyErr_Occurred()) {
+        PyErr_Format(PyExc_TypeError, "can't escape %s to binary",
+            Py_TYPE(self->wrapped)->tp_name);
+    }
+
+    return rv;
 }
 
 /* binary_str, binary_getquoted - return result of quoting */
 
 static PyObject *
-binary_str(binaryObject *self)
+binary_getquoted(binaryObject *self, PyObject *args)
 {
     if (self->buffer == NULL) {
-        binary_quote(self);
+        self->buffer = binary_quote(self);
     }
     Py_XINCREF(self->buffer);
     return self->buffer;
 }
 
 static PyObject *
-binary_getquoted(binaryObject *self, PyObject *args)
+binary_str(binaryObject *self)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-    return binary_str(self);
+    return psycopg_ensure_text(binary_getquoted(self, NULL));
 }
 
 static PyObject *
 binary_prepare(binaryObject *self, PyObject *args)
 {
-    connectionObject *conn;
+    PyObject *conn;
 
-    if (!PyArg_ParseTuple(args, "O", &conn))
+    if (!PyArg_ParseTuple(args, "O!", &connectionType, &conn))
         return NULL;
 
     Py_XDECREF(self->conn);
-    if (conn) {
-        self->conn = (PyObject*)conn;
-        Py_INCREF(self->conn);
-    }
+    self->conn = conn;
+    Py_INCREF(self->conn);
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static PyObject *
@@ -231,15 +183,15 @@ binary_conform(binaryObject *self, PyObject *args)
 /* object member list */
 
 static struct PyMemberDef binaryObject_members[] = {
-    {"adapted", T_OBJECT, offsetof(binaryObject, wrapped), RO},
-    {"buffer", T_OBJECT, offsetof(binaryObject, buffer), RO},
+    {"adapted", T_OBJECT, offsetof(binaryObject, wrapped), READONLY},
+    {"buffer", T_OBJECT, offsetof(binaryObject, buffer), READONLY},
     {NULL}
 };
 
 /* object method table */
 
 static PyMethodDef binaryObject_methods[] = {
-    {"getquoted", (PyCFunction)binary_getquoted, METH_VARARGS,
+    {"getquoted", (PyCFunction)binary_getquoted, METH_NOARGS,
      "getquoted() -> wrapped object value as SQL-quoted binary string"},
     {"prepare", (PyCFunction)binary_prepare, METH_VARARGS,
      "prepare(conn) -> prepare for binary encoding using conn"},
@@ -254,7 +206,7 @@ binary_setup(binaryObject *self, PyObject *str)
 {
     Dprintf("binary_setup: init binary object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
 
     self->buffer = NULL;
@@ -264,18 +216,7 @@ binary_setup(binaryObject *self, PyObject *str)
 
     Dprintf("binary_setup: good binary object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt);
-    return 0;
-}
-
-static int
-binary_traverse(PyObject *obj, visitproc visit, void *arg)
-{
-    binaryObject *self = (binaryObject *)obj;
-
-    Py_VISIT(self->wrapped);
-    Py_VISIT(self->buffer);
-    Py_VISIT(self->conn);
+        self, Py_REFCNT(self));
     return 0;
 }
 
@@ -290,10 +231,10 @@ binary_dealloc(PyObject* obj)
 
     Dprintf("binary_dealloc: deleted binary object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        obj, obj->ob_refcnt
+        obj, Py_REFCNT(obj)
       );
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -313,17 +254,6 @@ binary_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     return type->tp_alloc(type, 0);
 }
 
-static void
-binary_del(PyObject* self)
-{
-    PyObject_GC_Del(self);
-}
-
-static PyObject *
-binary_repr(binaryObject *self)
-{
-    return PyString_FromFormat("<psycopg2._psycopg.Binary object at %p>", self);
-}
 
 /* object type */
 
@@ -331,76 +261,41 @@ binary_repr(binaryObject *self)
 "Binary(buffer) -> new binary object"
 
 PyTypeObject binaryType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
-    "psycopg2._psycopg.Binary",
-    sizeof(binaryObject),
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "psycopg2.extensions.Binary",
+    sizeof(binaryObject), 0,
     binary_dealloc, /*tp_dealloc*/
     0,          /*tp_print*/
     0,          /*tp_getattr*/
     0,          /*tp_setattr*/
-
     0,          /*tp_compare*/
-    (reprfunc)binary_repr, /*tp_repr*/
+    0,          /*tp_repr*/
     0,          /*tp_as_number*/
     0,          /*tp_as_sequence*/
     0,          /*tp_as_mapping*/
     0,          /*tp_hash */
-
     0,          /*tp_call*/
     (reprfunc)binary_str, /*tp_str*/
     0,          /*tp_getattro*/
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
-
-    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
-
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
     binaryType_doc, /*tp_doc*/
-
-    binary_traverse, /*tp_traverse*/
+    0,          /*tp_traverse*/
     0,          /*tp_clear*/
-
     0,          /*tp_richcompare*/
     0,          /*tp_weaklistoffset*/
-
     0,          /*tp_iter*/
     0,          /*tp_iternext*/
-
-    /* Attribute descriptor and subclassing stuff */
-
     binaryObject_methods, /*tp_methods*/
     binaryObject_members, /*tp_members*/
     0,          /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-
     binary_init, /*tp_init*/
-    0, /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
+    0,          /*tp_alloc*/
     binary_new, /*tp_new*/
-    (freefunc)binary_del, /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
-    0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
-    0,          /*tp_cache*/
-    0,          /*tp_subclasses*/
-    0           /*tp_weaklist*/
 };
-
-
-/** module-level functions **/
-
-PyObject *
-psyco_Binary(PyObject *module, PyObject *args)
-{
-    PyObject *str;
-
-    if (!PyArg_ParseTuple(args, "O", &str))
-        return NULL;
-
-    return PyObject_CallFunction((PyObject *)&binaryType, "O", str);
-}
