@@ -40,13 +40,6 @@ typedef struct {
     PyObject *conn;
 } binaryObject;
 
-/* functions exported to psycopgmodule.c */
-
-HIDDEN PyObject *psyco_Binary(PyObject *module, PyObject *args);
-#define psyco_Binary_doc \
-    "Binary(buffer) -> new binary object\n\n" \
-    "Build an object capable to hold a binary string value."
-
 #ifdef __cplusplus
 }
 #endif
