@@ -113,11 +113,7 @@ exception_from_sqlstate(const char *sqlstate)
     case '4':
         switch (sqlstate[1]) {
         case '0': /* Class 40 - Transaction Rollback */
-#ifdef PSYCOPG_EXTENSIONS
             return TransactionRollbackError;
-#else
-            return OperationalError;
-#endif
         case '2': /* Class 42 - Syntax Error or Access Rule Violation */
         case '4': /* Class 44 - WITH CHECK OPTION Violation */
             return ProgrammingError;
@@ -129,11 +125,9 @@ exception_from_sqlstate(const char *sqlstate)
            Class 55 - Object Not In Prerequisite State
            Class 57 - Operator Intervention
            Class 58 - System Error (errors external to PostgreSQL itself) */
-#ifdef PSYCOPG_EXTENSIONS
         if (!strcmp(sqlstate, "57014"))
             return QueryCanceledError;
         else
-#endif
             return OperationalError;
     case 'F': /* Class F0 - Configuration File Error */
         return InternalError;
@@ -986,6 +980,10 @@ pq_execute(cursorObject *curs, const char *query, int async, int no_result, int
         }
         else {
             /* there was an error */
+            pthread_mutex_unlock(&(curs->conn->lock));
+            Py_BLOCK_THREADS;
+            PyErr_SetString(OperationalError,
+                            PQerrorMessage(curs->conn->pgconn));
             return -1;
         }
     }
